@@ -1,24 +1,21 @@
-// Copyright (C) 2022-2025 Hibiki AI Limited <info@hibiki-ai.com>
-//
-// This file is part of nanonext.
-//
-// nanonext is free software: you can redistribute it and/or modify it under the
-// terms of the GNU General Public License as published by the Free Software
-// Foundation, either version 3 of the License, or (at your option) any later
-// version.
-//
-// nanonext is distributed in the hope that it will be useful, but WITHOUT ANY
-// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License along with
-// nanonext. If not, see <https://www.gnu.org/licenses/>.
-
 // nanonext - C level - Synchronization Primitives and Signals -----------------
 
 #define NANONEXT_SIGNALS
 #include "nanonext.h"
 
+// internals -------------------------------------------------------------------
+
+static void nano_load_later(void) {
+
+  SEXP str, call;
+  PROTECT(str = Rf_mkString("later"));
+  PROTECT(call = Rf_lang2(Rf_install("loadNamespace"), str));
+  Rf_eval(call, R_BaseEnv);
+  UNPROTECT(2);
+  eln2 = (void (*)(void (*)(void *), void *, double, int)) R_GetCCallable("later", "execLaterNative2");
+
+}
+
 // aio completion callbacks ----------------------------------------------------
 
 static void sendaio_complete(void *arg) {
@@ -32,16 +29,38 @@ static void sendaio_complete(void *arg) {
 static void request_complete(void *arg) {
 
   nano_aio *raio = (nano_aio *) arg;
+  nano_saio *saio = (nano_saio *) raio->cb;
   int res = nng_aio_result(raio->aio);
   if (res == 0) {
     nng_msg *msg = nng_aio_get_msg(raio->aio);
     raio->data = msg;
     nng_pipe p = nng_msg_get_pipe(msg);
     res = - (int) p.id;
+  } else if (res == 5 && saio->id) {
+    nng_msg *msg = NULL;
+    if (nng_msg_alloc(&msg, 0) == 0) {
+      if (nng_msg_append_u32(msg, 0) ||
+          nng_msg_append(msg, &saio->id, sizeof(int)) ||
+          nng_ctx_sendmsg(*saio->ctx, msg, 0)) {
+        nng_msg_free(msg);
+      }
+    }
+  }
+
+  if (raio->next != NULL) {
+    nano_cv *ncv = (nano_cv *) raio->next;
+    nng_cv *cv = ncv->cv;
+    nng_mtx *mtx = ncv->mtx;
+
+    nng_mtx_lock(mtx);
+    raio->result = res;
+    ncv->condition++;
+    nng_cv_wake(cv);
+    nng_mtx_unlock(mtx);
+  } else {
+    raio->result = res;
   }
-  raio->result = res;
 
-  nano_saio *saio = (nano_saio *) raio->cb;
   if (saio->cb != NULL)
     later2(raio_invoke_cb, saio->cb);
 
@@ -66,33 +85,6 @@ static void request_complete_dropcon(void *arg) {
 
 }
 
-static void request_complete_signal(void *arg) {
-
-  nano_aio *raio = (nano_aio *) arg;
-  nano_cv *ncv = (nano_cv *) raio->next;
-  nng_cv *cv = ncv->cv;
-  nng_mtx *mtx = ncv->mtx;
-
-  int res = nng_aio_result(raio->aio);
-  if (res == 0) {
-    nng_msg *msg = nng_aio_get_msg(raio->aio);
-    raio->data = msg;
-    nng_pipe p = nng_msg_get_pipe(msg);
-    res = - (int) p.id;
-  }
-
-  nng_mtx_lock(mtx);
-  raio->result = res;
-  ncv->condition++;
-  nng_cv_wake(cv);
-  nng_mtx_unlock(mtx);
-
-  nano_saio *saio = (nano_saio *) raio->cb;
-  if (saio->cb != NULL)
-    later2(raio_invoke_cb, saio->cb);
-
-}
-
 void pipe_cb_signal(nng_pipe p, nng_pipe_ev ev, void *arg) {
 
   int sig;
@@ -110,6 +102,8 @@ void pipe_cb_signal(nng_pipe p, nng_pipe_ev ev, void *arg) {
 #ifdef _WIN32
     raise(sig);
 #else
+    if (sig == SIGINT)
+      R_interrupts_pending = 1;
     kill(getpid(), sig);
 #endif
 
@@ -132,7 +126,13 @@ static void pipe_cb_monitor(nng_pipe p, nng_pipe_ev ev, void *arg) {
   nng_mtx_lock(mtx);
   if (monitor->updates >= monitor->size) {
     monitor->size += 8;
-    monitor->ids = R_Realloc(monitor->ids, monitor->size, int);
+    int *ids = realloc(monitor->ids, monitor->size * sizeof(int));
+    if (ids == NULL) {
+      monitor->size -= 8;
+      nng_mtx_unlock(mtx);
+      return;
+    }
+    monitor->ids = ids;
   }
   monitor->ids[monitor->updates] = ev == NNG_PIPE_EV_ADD_POST ? id : -id;
   monitor->updates++;
@@ -150,7 +150,7 @@ static void cv_finalizer(SEXP xptr) {
   nano_cv *xp = (nano_cv *) NANO_PTR(xptr);
   nng_cv_free(xp->cv);
   nng_mtx_free(xp->mtx);
-  R_Free(xp);
+  free(xp);
 
 }
 
@@ -163,8 +163,8 @@ static void request_finalizer(SEXP xptr) {
   nng_aio_free(xp->aio);
   if (xp->data != NULL)
     nng_msg_free((nng_msg *) xp->data);
-  R_Free(saio);
-  R_Free(xp);
+  free(saio);
+  free(xp);
 
 }
 
@@ -172,8 +172,8 @@ static void monitor_finalizer(SEXP xptr) {
 
   if (NANO_PTR(xptr) == NULL) return;
   nano_monitor *xp = (nano_monitor *) NANO_PTR(xptr);
-  R_Free(xp->ids);
-  R_Free(xp);
+  free(xp->ids);
+  free(xp);
 
 }
 
@@ -181,15 +181,16 @@ static void monitor_finalizer(SEXP xptr) {
 
 SEXP rnng_cv_alloc(void) {
 
-  nano_cv *cvp = R_Calloc(1, nano_cv);
   SEXP xp;
   int xc;
+  nano_cv *cvp = calloc(1, sizeof(nano_cv));
+  NANO_ENSURE_ALLOC(cvp);
 
   if ((xc = nng_mtx_alloc(&cvp->mtx)))
-    goto exitlevel1;
+    goto fail;
 
   if ((xc = nng_cv_alloc(&cvp->cv, cvp->mtx)))
-    goto exitlevel2;
+    goto fail;
 
   PROTECT(xp = R_MakeExternalPtr(cvp, nano_CvSymbol, R_NilValue));
   R_RegisterCFinalizerEx(xp, cv_finalizer, TRUE);
@@ -198,10 +199,10 @@ SEXP rnng_cv_alloc(void) {
   UNPROTECT(1);
   return xp;
 
-  exitlevel2:
-    nng_mtx_free(cvp->mtx);
-  exitlevel1:
-    R_Free(cvp);
+  fail:
+  nng_mtx_free(cvp->mtx);
+  free(cvp);
+  failmem:
   ERROR_OUT(xc);
 
 }
@@ -209,7 +210,7 @@ SEXP rnng_cv_alloc(void) {
 SEXP rnng_cv_wait(SEXP cvar) {
 
   if (NANO_PTR_CHECK(cvar, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
 
   nano_cv *ncv = (nano_cv *) NANO_PTR(cvar);
   nng_cv *cv = ncv->cv;
@@ -230,7 +231,7 @@ SEXP rnng_cv_wait(SEXP cvar) {
 SEXP rnng_cv_until(SEXP cvar, SEXP msec) {
 
   if (NANO_PTR_CHECK(cvar, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
 
   nano_cv *ncv = (nano_cv *) NANO_PTR(cvar);
   nng_cv *cv = ncv->cv;
@@ -269,7 +270,7 @@ SEXP rnng_cv_until(SEXP cvar, SEXP msec) {
 SEXP rnng_cv_wait_safe(SEXP cvar) {
 
   if (NANO_PTR_CHECK(cvar, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
 
   nano_cv *ncv = (nano_cv *) NANO_PTR(cvar);
   nng_cv *cv = ncv->cv;
@@ -304,7 +305,7 @@ SEXP rnng_cv_wait_safe(SEXP cvar) {
 SEXP rnng_cv_until_safe(SEXP cvar, SEXP msec) {
 
   if (NANO_PTR_CHECK(cvar, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
 
   nano_cv *ncv = (nano_cv *) NANO_PTR(cvar);
   nng_cv *cv = ncv->cv;
@@ -353,7 +354,7 @@ SEXP rnng_cv_until_safe(SEXP cvar, SEXP msec) {
 SEXP rnng_cv_reset(SEXP cvar) {
 
   if (NANO_PTR_CHECK(cvar, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
 
   nano_cv *ncv = (nano_cv *) NANO_PTR(cvar);
   nng_mtx *mtx = ncv->mtx;
@@ -370,7 +371,8 @@ SEXP rnng_cv_reset(SEXP cvar) {
 SEXP rnng_cv_value(SEXP cvar) {
 
   if (NANO_PTR_CHECK(cvar, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
+
   nano_cv *ncv = (nano_cv *) NANO_PTR(cvar);
   nng_mtx *mtx = ncv->mtx;
   int cond;
@@ -385,7 +387,7 @@ SEXP rnng_cv_value(SEXP cvar) {
 SEXP rnng_cv_signal(SEXP cvar) {
 
   if (NANO_PTR_CHECK(cvar, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
 
   nano_cv *ncv = (nano_cv *) NANO_PTR(cvar);
   nng_cv *cv = ncv->cv;
@@ -402,14 +404,16 @@ SEXP rnng_cv_signal(SEXP cvar) {
 
 // request ---------------------------------------------------------------------
 
-SEXP rnng_request(SEXP con, SEXP data, SEXP sendmode, SEXP recvmode, SEXP timeout, SEXP cvar, SEXP clo) {
+SEXP rnng_request(SEXP con, SEXP data, SEXP sendmode, SEXP recvmode, SEXP timeout, SEXP cvar, SEXP msgid, SEXP clo) {
 
   if (NANO_PTR_CHECK(con, nano_ContextSymbol))
-    Rf_error("'con' is not a valid Context");
+    Rf_error("`con` is not a valid Context");
 
   const nng_duration dur = timeout == R_NilValue ? NNG_DURATION_DEFAULT : (nng_duration) nano_integer(timeout);
   const uint8_t mod = (uint8_t) nano_matcharg(recvmode);
-  int signal, drop;
+  const int raw = nano_encode_mode(sendmode);
+  const int id = msgid != R_NilValue ? NANO_INTEGER(msgid) : 0;
+  int signal, drop, xc;
   if (cvar == R_NilValue) {
     signal = 0;
     drop = 0;
@@ -417,40 +421,48 @@ SEXP rnng_request(SEXP con, SEXP data, SEXP sendmode, SEXP recvmode, SEXP timeou
     signal = !NANO_PTR_CHECK(cvar, nano_CvSymbol);
     drop = 1 - signal;
   }
+
+  nano_saio *saio = NULL;
+  nano_aio *raio = NULL;
+  nng_msg *msg = NULL;
+  SEXP aio, env, fun;
+  nano_buf buf;
+
+  if (raw) {
+    nano_encode(&buf, data);
+  } else {
+    nano_serialize(&buf, data, NANO_PROT(con));
+  }
+
+  saio = calloc(1, sizeof(nano_saio));
+  NANO_ENSURE_ALLOC(saio);
+
+  raio = calloc(1, sizeof(nano_aio));
+  NANO_ENSURE_ALLOC(raio);
+
   nng_ctx *ctx = (nng_ctx *) NANO_PTR(con);
   nano_cv *ncv = signal ? (nano_cv *) NANO_PTR(cvar) : NULL;
 
-  SEXP aio, env, fun;
-  nano_buf buf;
-  nano_saio *saio;
-  nano_aio *raio;
-  nng_msg *msg;
-  int xc;
+  saio->ctx = ctx;
+  saio->id = id;
 
-  nano_encodes(sendmode) == 2 ? nano_encode(&buf, data) : nano_serialize(&buf, data, NANO_PROT(con));
-  saio = R_Calloc(1, nano_saio);
-  saio->cb = NULL;
-
-  if ((xc = nng_msg_alloc(&msg, 0)))
-    goto exitlevel1;
-
-  if ((xc = nng_msg_append(msg, buf.buf, buf.cur)) ||
+  if ((xc = nng_msg_alloc(&msg, 0)) ||
+      (xc = nng_msg_append(msg, buf.buf, buf.cur)) ||
       (xc = nng_aio_alloc(&saio->aio, sendaio_complete, saio))) {
     nng_msg_free(msg);
-    goto exitlevel1;
+    goto fail;
   }
 
   nng_aio_set_msg(saio->aio, msg);
   nng_ctx_send(*ctx, saio->aio);
 
-  raio = R_Calloc(1, nano_aio);
   raio->type = signal ? REQAIOS : REQAIO;
   raio->mode = mod;
   raio->cb = saio;
   raio->next = ncv;
 
-  if ((xc = nng_aio_alloc(&raio->aio, signal ? request_complete_signal : drop ? request_complete_dropcon : request_complete, raio)))
-    goto exitlevel2;
+  if ((xc = nng_aio_alloc(&raio->aio, drop ? request_complete_dropcon : request_complete, raio)))
+    goto fail;
 
   nng_aio_set_timeout(raio->aio, dur);
   nng_ctx_recv(*ctx, raio->aio);
@@ -458,6 +470,8 @@ SEXP rnng_request(SEXP con, SEXP data, SEXP sendmode, SEXP recvmode, SEXP timeou
 
   PROTECT(aio = R_MakeExternalPtr(raio, nano_AioSymbol, NANO_PROT(con)));
   R_RegisterCFinalizerEx(aio, request_finalizer, TRUE);
+  Rf_setAttrib(aio, nano_ContextSymbol, con);
+  Rf_setAttrib(aio, nano_IdSymbol, Rf_ScalarInteger(id));
 
   PROTECT(env = R_NewEnv(R_NilValue, 0, 0));
   Rf_classgets(env, nano_reqAio);
@@ -469,11 +483,11 @@ SEXP rnng_request(SEXP con, SEXP data, SEXP sendmode, SEXP recvmode, SEXP timeou
   UNPROTECT(3);
   return env;
 
-  exitlevel2:
-  R_Free(raio);
+  fail:
   nng_aio_free(saio->aio);
-  exitlevel1:
-  R_Free(saio);
+  failmem:
+  free(raio);
+  free(saio);
   NANO_FREE(buf);
   return mk_error_data(xc);
 
@@ -490,14 +504,8 @@ SEXP rnng_set_promise_context(SEXP x, SEXP ctx) {
 
   nano_aio *raio = (nano_aio *) NANO_PTR(aio);
 
-  if (eln2 == NULL) {
-    SEXP str, call;
-    PROTECT(str = Rf_mkString("later"));
-    PROTECT(call = Rf_lang2(Rf_install("loadNamespace"), str));
-    Rf_eval(call, R_BaseEnv);
-    UNPROTECT(2);
-    eln2 = (void (*)(void (*)(void *), void *, double, int)) R_GetCCallable("later", "execLaterNative2");
-  }
+  if (eln2 == NULL)
+    nano_load_later();
 
   switch (raio->type) {
   case REQAIO:
@@ -528,7 +536,7 @@ SEXP rnng_set_promise_context(SEXP x, SEXP ctx) {
 SEXP rnng_pipe_notify(SEXP socket, SEXP cv, SEXP add, SEXP remove, SEXP flag) {
 
   if (NANO_PTR_CHECK(socket, nano_SocketSymbol))
-    Rf_error("'socket' is not a valid Socket");
+    Rf_error("`socket` is not a valid Socket");
 
   int xc;
   nng_socket *sock;
@@ -545,7 +553,7 @@ SEXP rnng_pipe_notify(SEXP socket, SEXP cv, SEXP add, SEXP remove, SEXP flag) {
     return nano_success;
 
   } else if (NANO_PTR_CHECK(cv, nano_CvSymbol)) {
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
   }
 
   sock = (nng_socket *) NANO_PTR(socket);
@@ -560,6 +568,7 @@ SEXP rnng_pipe_notify(SEXP socket, SEXP cv, SEXP add, SEXP remove, SEXP flag) {
   if (NANO_INTEGER(remove) && (xc = nng_pipe_notify(*sock, NNG_PIPE_EV_REM_POST, pipe_cb_signal, cvp)))
     ERROR_OUT(xc);
 
+  R_MakeWeakRef(socket, cv, R_NilValue, FALSE);
   return nano_success;
 
 }
@@ -569,26 +578,28 @@ SEXP rnng_pipe_notify(SEXP socket, SEXP cv, SEXP add, SEXP remove, SEXP flag) {
 SEXP rnng_monitor_create(SEXP socket, SEXP cv) {
 
   if (NANO_PTR_CHECK(socket, nano_SocketSymbol))
-    Rf_error("'socket' is not a valid Socket");
+    Rf_error("`socket` is not a valid Socket");
 
   if (NANO_PTR_CHECK(cv, nano_CvSymbol))
-    Rf_error("'cv' is not a valid Condition Variable");
+    Rf_error("`cv` is not a valid Condition Variable");
 
   const int n = 8;
-  nano_monitor *monitor = R_Calloc(1, nano_monitor);
-  monitor->ids = R_Calloc(n, int);
+  SEXP xptr;
+  int xc;
+
+  nano_monitor *monitor = calloc(1, sizeof(nano_monitor));
+  NANO_ENSURE_ALLOC(monitor);
+  monitor->ids = calloc(n, sizeof(int));
+  NANO_ENSURE_ALLOC(monitor->ids);
   monitor->size = n;
   monitor->cv = (nano_cv *) NANO_PTR(cv);
   nng_socket *sock = (nng_socket *) NANO_PTR(socket);
 
-  SEXP xptr;
-  int xc;
-
   if ((xc = nng_pipe_notify(*sock, NNG_PIPE_EV_ADD_POST, pipe_cb_monitor, monitor)))
-    ERROR_OUT(xc);
+    goto fail;
 
   if ((xc = nng_pipe_notify(*sock, NNG_PIPE_EV_REM_POST, pipe_cb_monitor, monitor)))
-    ERROR_OUT(xc);
+    goto fail;
 
   PROTECT(xptr = R_MakeExternalPtr(monitor, nano_MonitorSymbol, R_NilValue));
   R_RegisterCFinalizerEx(xptr, monitor_finalizer, TRUE);
@@ -598,12 +609,19 @@ SEXP rnng_monitor_create(SEXP socket, SEXP cv) {
 
   return xptr;
 
+  fail:
+  failmem:
+  if (monitor != NULL)
+    free(monitor->ids);
+  free(monitor);
+  ERROR_OUT(xc);
+
 }
 
 SEXP rnng_monitor_read(SEXP x) {
 
   if (NANO_PTR_CHECK(x, nano_MonitorSymbol))
-    Rf_error("'x' is not a valid Monitor");
+    Rf_error("`x` is not a valid Monitor");
 
   nano_monitor *monitor = (nano_monitor *) NANO_PTR(x);
 
