@@ -70,11 +70,15 @@ test_equal(n1$dialer[[1]]$state, "started")
 
 test_error(n$send(list(), mode = "raw"), "atomic vector type")
 test_error(n$recv(mode = "none"), "mode")
-test_error(n$recv(mode = "c"), "mode")
+test_error(n$recv(mode = "int"), "mode")
+test_error(n$recv(mode = "logica"), "mode")
+test_error(n$recv(mode = "charact"), "mode")
+test_error(n$recv(mode = "positrons"), "mode")
 test_true(is_aio(raio <- n1$recv_aio(timeout = 1L)))
 test_print(raio)
 test_class("errorValue", call_aio(raio)$data)
 test_class("errorValue", raio$data)
+test_zero(pipe_id(raio))
 r <- n$send(data.frame(), block = FALSE)
 if (r == 8L) r <- n$send(data.frame(), block = 500L)
 test_zero(r)
@@ -83,17 +87,20 @@ if (is_error_value(r)) r <- n1$recv(block = 500)
 test_class("data.frame", r)
 test_zero(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
 test_identical(n$recv("character", block = 500), c("test", "", "spec"))
-test_zero(n$send(1:5, mode = "r"))
-test_equal(length(n1$recv("int", block = 500)), 5L)
+test_zero(n$send(1:5, mode = "raw"))
+test_equal(length(n1$recv("integer", block = 500)), 5L)
 test_true(is_aio(saio <- n1$send_aio(paste(replicate(5, random(1e3L)), collapse = ""), mode = 1L, timeout = 900)))
 test_print(saio)
 if (later) test_null(.keep(saio, new.env()))
 test_class("sendAio", call_aio(saio))
 test_zero(saio$result)
 test_error(n$send("wrong mode", mode = "none"), "mode")
+test_error(n$send("wrong mode", mode = "ser"), "mode")
+test_error(n$send("wrong mode", mode = "rawial"), "mode")
 test_class("recvAio", raio <- n$recv_aio(timeout = 500))
 test_print(raio)
 test_equal(nchar(call_aio(raio)[["value"]]), 10000L)
+test_type("integer", pipe_id(raio))
 raio$newfield <- "doesnotwork"
 test_null(raio$newfield)
 test_class("sendAio", saio <- n$send_aio(c(1.1, 2.2), mode = "raw", timeout = 500))
@@ -101,7 +108,7 @@ saio$newfield <- "doesnotwork"
 test_null(saio$newfield)
 test_type("logical", unresolved(saio))
 test_type("logical", .unresolved(saio))
-test_class("recvAio", msg <- n1$recv_aio(mode = "numer", timeout = 500))
+test_class("recvAio", msg <- n1$recv_aio(mode = "numeric", timeout = 500))
 test_identical(call_aio(msg), msg)
 test_class("recvAio", msg <- n1$recv_aio(mode = "complex", timeout = 500))
 test_null(stop_aio(msg))
@@ -109,29 +116,29 @@ test_null(stop_aio(n))
 test_identical(call_aio(msg), msg)
 test_class("errorValue", msg$data)
 test_identical(call_aio(n), n)
-test_class("sendAio", sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500))
+test_class("sendAio", sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 test_class("recvAio", rraio <- n1$recv_aio(mode = 1L, timeout = 500))
 test_true(is_nul_byte(suppressWarnings(call_aio_(rraio)$data)))
-test_class("sendAio", sraio <- n$send_aio(as.raw(1L), mode = "ra", timeout = 500))
+test_class("sendAio", sraio <- n$send_aio(as.raw(1L), mode = 2L, timeout = 500))
 test_class("recvAio", rraio <- n1$recv_aio(mode = "raw", timeout = 500))
 test_type("raw", call_aio(rraio)$data)
 test_class("sendAio", sraio <- n$send_aio(c(1+2i, 4+3i), mode = "raw", timeout = 500))
-test_class("recvAio", rraio <- n1$recv_aio(mode = "co", timeout = 500))
+test_class("recvAio", rraio <- n1$recv_aio(mode = "complex", timeout = 500))
 test_type("complex", call_aio(rraio)$data)
 test_class("sendAio", sraio <- n$send_aio(5, mode = "raw", timeout = 500))
-test_class("recvAio", rraio <- n1$recv_aio(mode = "d", timeout = 500))
+test_class("recvAio", rraio <- n1$recv_aio(mode = "double", timeout = 500))
 test_type("double", call_aio(rraio)$data)
 test_class("sendAio", sraio <- n$send_aio(c(1, 2), mode = "raw", timeout = 500))
-test_class("recvAio", rraio <- n1$recv_aio(mode = "n", timeout = 500))
+test_class("recvAio", rraio <- n1$recv_aio(mode = "numeric", timeout = 500))
 test_true(is.numeric(call_aio(rraio)$data))
 test_class("sendAio", sraio <- n$send_aio(c(1L, 2L, 3L), mode = "raw", timeout = 500))
-test_class("recvAio", rraio <- n1$recv_aio(mode = "i", timeout = 500))
+test_class("recvAio", rraio <- n1$recv_aio(mode = "integer", timeout = 500))
 test_type("integer", call_aio(rraio)$data)
 test_class("sendAio", sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 test_class("recvAio", rraio <- n1$recv_aio(mode = "double", timeout = 500))
 test_type("raw", suppressWarnings(call_aio(rraio)$data))
 test_class("sendAio", sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
-test_class("recvAio", rraio <- n1$recv_aio(mode = "int", timeout = 500))
+test_class("recvAio", rraio <- n1$recv_aio(mode = "integer", timeout = 500))
 test_type("raw", suppressWarnings(call_aio(rraio)$data))
 test_class("sendAio", sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 test_class("recvAio", rraio <- n1$recv_aio(mode = "logical", timeout = 500))
@@ -206,20 +213,28 @@ test_error(req$opt("false", list()), "type")
 
 test_class("nanoContext", ctx <- context(rep))
 test_print(ctx)
+test_equal(.header(12345L), 12345L)
 test_true(.mark())
-test_class("sendAio", csaio <- req$send_aio(data.frame(), mode = "seria", timeout = 500))
-test_true(!.mark(FALSE))
+test_class("sendAio", csaio <- req$send_aio(data.frame(), mode = 1L, timeout = 500))
 test_zero(call_aio_(csaio)$result)
-test_class("recvAio", craio <- recv_aio(ctx, timeout = 500))
-test_type("list", collect_aio(craio))
+test_class("recvAio", craio <- recv_aio(ctx, mode = 8L, timeout = 500))
+test_type("raw", res <- collect_aio(craio))
+test_true(.read_marker(res))
+test_true(!.read_marker("not"))
+test_equal(.read_header(res), 12345L)
+test_equal(.read_header("not"), 0L)
+test_type("list", unserialize(res[9:length(res)]))
+test_equal(.header(0L), 0L)
+test_true(!.mark(FALSE))
 test_zero(req$send("context test", mode ="raw", block = 500))
 test_equal(recv(ctx, mode = "string", block = 500), "context test")
-test_type("integer", req$send(data.frame(), mode = "seri", block = 500))
-test_class("recvAio", msg <- recv_aio(ctx, mode = "ser", timeout = 500))
+test_type("integer", req$send(data.frame(), mode = "serial", block = 500))
+test_class("recvAio", msg <- recv_aio(ctx, mode = "serial", timeout = 500))
 test_type("logical", .unresolved(msg))
 test_type("logical", unresolved(msg))
 test_class("data.frame", call_aio(msg)$data)
 test_true(!unresolved(msg))
+test_equal(.header(2025250L), 2025250L)
 test_zero(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500))
 test_class("recvAio", msg <- recv_aio(ctx, mode = 6L, timeout = 500))
 test_type("logical", msg[])
@@ -239,38 +254,37 @@ test_null(call_aio(recv_aio(ctx, mode = 1L, timeout = 500))[["value"]])
 test_class("sendAio", saio <- send_aio(ctx, as.raw(1L), mode = 2L, timeout = 500))
 test_identical(req$recv(mode = 8L, block = 500), as.raw(1L))
 test_class("recvAio", rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode = "complex", timeout = 500))
-test_zero(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500))
+test_zero(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "raw", timeout = 500))
 test_type("complex", call_aio(rek)[["data"]])
 test_class("recvAio", rek <- request(req$context, c(1+3i, 4+2i), send_mode = "serial", recv_mode = "serial", timeout = 500))
 test_zero(reply(ctx, execute = identity, recv_mode = 1L, send_mode = 1L, timeout = 500))
 test_type("complex", call_aio(rek)[["data"]])
 test_type("integer", rek[["aio"]])
+test_equal(.header(0L), 0L)
 
-test_type("list", cfg <- serial_config(class = "custom", sfunc = function(x) raw(1L), ufunc = as.integer, vec = FALSE))
-test_equal(length(cfg), 4L)
-test_type("closure", cfg[[2L]])
+test_type("list", cfg <- serial_config(class = c("invalid", "custom"), sfunc = list(identity, function(x) raw(1L)), ufunc = list(identity, as.integer)))
 opt(req$socket, "serial") <- cfg
 opt(rep, "serial") <- cfg
 custom <- list(`class<-`(new.env(), "custom"), new.env())
 test_zero(send(req$socket, custom, mode = "serial", block = 500))
 test_type("integer", recv(rep, block = 500)[[1L]])
-cfg <- serial_config("custom", function(x) as.raw(length(x)), function(x) lapply(seq_len(as.integer(x)), new.env), vec = NA)
-test_true(cfg[[4L]])
+custom <- list(`class<-`(new.env(), "unused"), new.env())
+test_zero(send(req$socket, custom, mode = "serial", block = 500))
+test_type("list", recv(rep, block = 500))
+cfg <- serial_config("custom", function(x) as.raw(length(x)), function(x) lapply(seq_len(as.integer(x)), new.env))
+test_type("list", cfg)
 opt(req$socket, "serial") <- cfg
 opt(rep, "serial") <- cfg
 test_zero(send(rep, custom, block = 500))
 test_type("list", recv(req$socket, mode = 1L, block = 500))
 opt(req$socket, "serial") <- list()
 opt(rep, "serial") <- list()
-test_error(serial_config(1L, identity, identity), "must be a character string")
-test_error(serial_config("custom", "func1", "func2"), "must be functions")
-test_error(serial_config("class", identity, identity, "wrong"), "must be a logical value")
+test_error(serial_config(1L, identity, identity), "must be a character vector")
+test_error(serial_config(c("custom", "custom2"), list(identity), list(identity)), "must all be the same length")
+test_error(serial_config("custom", "func1", "func2"), "must be a function or list of functions")
+test_error(serial_config("custom", identity, "func2"), "must be a function or list of functions")
 test_error(opt(rep, "wrong") <- cfg, "not supported")
 test_error(opt(rep, "serial") <- pairlist(a = 1L), "not supported")
-test_error(opt(rep, "serial") <- list("wrong"), "Invalid argument")
-test_error(opt(rep, "serial") <- list("class", identity, identity, "wrong"), "Invalid argument")
-test_error(opt(rep, "serial") <- list("class", "wrong", function() {}, NA), "Invalid argument")
-test_error(opt(rep, "serial") <- list("class", function() {}, "wrong", NA), "Invalid argument")
 
 test_class("recvAio", cs <- request(req$context, "test", send_mode = "serial", cv = cv, timeout = 500))
 test_notnull(cs$data)
@@ -284,6 +298,7 @@ test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, cv = NA))
 test_notnull(cs$data)
 test_true(recv(ctxn, block = 500))
 test_zero(send(ctxn, TRUE, mode = 1L, block = 500))
+test_class("recvAio", cs <- request(.context(req$socket), data = TRUE, timeout = 5, id = 12L))
 test_zero(reap(ctxn))
 test_equal(reap(ctxn), 7L)
 test_zero(pipe_notify(rep, cv, add = TRUE, flag = TRUE))
@@ -373,6 +388,9 @@ test_class("nanoMonitor", m <- monitor(poly, cv))
 test_print(m)
 test_zero(listen(poly))
 test_null(read_monitor(m))
+test_class("sendAio", send_aio(poly, "one", timeout = 50))
+invisible(gc())
+test_null(msleep(55))
 test_zero(dial(poly1))
 test_zero(dial(poly2))
 test_true(wait(cv))
@@ -397,8 +415,12 @@ test_class("nanoSocket", push <- socket(protocol = "push"))
 test_class("nanoSocket", pull <- socket(protocol = "pull"))
 test_class("nanoSocket", pair <- socket(protocol = "pair"))
 test_class("nano", bus)
-test_equal(suppressWarnings(listen(bus, url = "test")), 3L)
-test_equal(suppressWarnings(dial(bus, url = "test")), 3L)
+test_equal(suppressWarnings(listen(bus, url = "test", fail = "warn")), 3L)
+test_error(listen(bus, url = "test", fail = "error"), "argument")
+test_equal(listen(bus, url = "test", fail = "none"), 3L)
+test_equal(suppressWarnings(dial(bus, url = "test", fail = 1L)), 3L)
+test_error(dial(bus, url = "test", fail = 2L), "argument")
+test_equal(dial(bus, url = "test", fail = 3L), 3L)
 test_error(listen(bus, url = "tls+tcp://localhost/:0", tls = "wrong"), "valid TLS")
 test_error(dial(bus, url = "tls+tcp://localhost/:0", tls = "wrong"), "valid TLS")
 test_zero(close(bus))
@@ -430,7 +452,7 @@ test_null(stop_aio(put1))
 test_class("ncurlAio", haio <- ncurl_aio("https://i.i"))
 test_class("errorValue", call_aio(haio)$data)
 test_print(haio$data)
-test_class("ncurlAio", ncaio <- ncurl_aio("https://shikokuchuo.net/nanonext/reference/figures/logo.png"))
+test_class("ncurlAio", ncaio <- ncurl_aio("https://nanonext.r-lib.org/reference/figures/logo.png"))
 if (suppressWarnings(call_aio(ncaio)$status == 200L)) test_type("raw", ncaio$data)
 test_class("errorValue", ncurl_aio("http")$data)
 sess <- ncurl_session("https://postman-echo.com/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = c("date", "Server"), timeout = 3000L)
@@ -484,34 +506,30 @@ for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511, 600)
   test_type("character", status_code(i))
 
 s <- tryCatch(stream(dial = "wss://echo.websocket.events/", textframes = TRUE), error = function(e) NULL)
-is_nano(s) && {
-  test_notnull(recv(s, block = 500L))
-  test_type("character", opt(s, "ws:response-headers"))
-  test_error(opt(s, "ws:request-headers") <- "test\n", 24)
-  test_type("integer", send(s, c("message1", "test"), block = 500L))
-  test_notnull(recv(s, block = FALSE))
-  test_type("integer", send(s, "message2", block = FALSE))
-  test_notnull(suppressWarnings(recv(s, mode = 9L, block = 100)))
-  test_type("integer", send(s, 2L, block = 500))
-  test_class("recvAio", sr <- recv_aio(s, mode = "i", timeout = 500L, n = 8192L))
-  test_notnull(suppressWarnings(call_aio(sr)[["data"]]))
-  test_null(stop_aio(sr))
-  test_class("sendAio", ss <- send_aio(s, "async", timeout = 500L))
-  test_type("integer", ss[])
-  test_null(stop_aio(ss))
-  test_type("integer", send(s, 12.56, mode = "raw", block = 500L))
-  test_class("recvAio", sr <- recv_aio(s, mode = "double", timeout = 500L, cv = cv))
-  test_notnull(suppressWarnings(call_aio_(sr)[["data"]]))
-  test_true(cv_value(cv) > 0L)
-  test_type("character", opt(s, "ws:request-headers"))
-  test_notnull(opt(s, "tcp-nodelay") <- FALSE)
-  test_error(recv(s, mode = "none", block = FALSE), "should be one of character")
-  test_error(recv(s, mode = "c", block = FALSE), "should be one of character")
-  test_error(opt(s, "none"), "supported")
-  test_error(`opt<-`(s, "none", list()), "supported")
-  test_print(s)
-  test_type("integer", close(s))
-}
+if (is_nano(s)) test_notnull(recv(s, block = 500L))
+if (is_nano(s)) test_type("character", opt(s, "ws:response-headers"))
+if (is_nano(s)) test_error(opt(s, "ws:request-headers") <- "test\n", 24)
+if (is_nano(s)) test_type("integer", send(s, c("message1", "test"), block = 500L))
+if (is_nano(s)) test_notnull(recv(s, block = FALSE))
+if (is_nano(s)) test_type("integer", send(s, "message2", block = FALSE))
+if (is_nano(s)) test_notnull(suppressWarnings(recv(s, mode = 9L, block = 100)))
+if (is_nano(s)) test_type("integer", send(s, 2L, block = 500))
+if (is_nano(s)) test_class("recvAio", sr <- recv_aio(s, mode = "integer", timeout = 500L, n = 8192L))
+if (is_nano(s)) test_notnull(suppressWarnings(call_aio(sr)[["data"]]))
+if (is_nano(s)) test_null(stop_aio(sr))
+if (is_nano(s)) test_class("sendAio", ss <- send_aio(s, "async", timeout = 500L))
+if (is_nano(s)) test_type("integer", ss[])
+if (is_nano(s)) test_null(stop_aio(ss))
+if (is_nano(s)) test_type("integer", send(s, 12.56, mode = "raw", block = 500L))
+if (is_nano(s)) test_class("recvAio", sr <- recv_aio(s, mode = "double", timeout = 500L, cv = cv))
+if (is_nano(s)) test_notnull(suppressWarnings(call_aio_(sr)[["data"]]))
+if (is_nano(s)) test_true(cv_value(cv) > 0L)
+if (is_nano(s)) test_type("character", opt(s, "ws:request-headers"))
+if (is_nano(s)) test_notnull(opt(s, "tcp-nodelay") <- FALSE)
+if (is_nano(s)) test_error(opt(s, "none"), "supported")
+if (is_nano(s)) test_error(`opt<-`(s, "none", list()), "supported")
+if (is_nano(s)) test_print(s)
+if (is_nano(s)) test_type("integer", close(s))
 
 test_equal(nanonext:::.DollarNames.ncurlAio(NULL, "sta"), "status")
 test_equal(nanonext:::.DollarNames.recvAio(NULL, "dat"), "data")
@@ -597,8 +615,8 @@ test_true(is_ncurl_session(sess) || is_error_value(sess))
 if (is_ncurl_session(sess)) test_class("errorValue", transact(sess)[["headers"]])
 test_type("externalptr", s <- socket(listen = "tls+tcp://127.0.0.1:5556", tls = tls_config(server = cert$server)))
 test_type("externalptr", s1 <- socket(dial = "tls+tcp://127.0.0.1:5556", tls = tls))
-test_true(suppressWarnings(dial(s, url = "tls+tcp://.", tls = tls, error = FALSE)) > 0)
-test_true(suppressWarnings(listen(s, url = "tls+tcp://.", tls = tls, error = FALSE)) > 0)
+test_true(suppressWarnings(dial(s, url = "tls+tcp://.", tls = tls)) > 0)
+test_true(suppressWarnings(listen(s, url = "tls+tcp://.", tls = tls)) > 0)
 test_zero(close(s1))
 test_zero(close(s))
 if (promises) test_class("nano", s <- socket(listen = "inproc://nanonext"))
@@ -616,19 +634,34 @@ if (promises) test_true(is_aio(n <- ncurl_aio("https://postman-echo.com/get")))
 if (promises) test_true(promises::is.promise(promises::then(n, identity, identity)))
 if (promises) test_true(promises::is.promising(call_aio(n)))
 if (promises) test_true(promises::is.promise(promises::as.promise(call_aio(ncurl_aio("https://postman-echo.com/get")))))
-if (promises) later::run_now()
+if (promises) later::run_now(1L)
 if (promises) test_zero(close(s1))
 if (promises) test_zero(close(s))
-if (promises) later::run_now()
-
+if (promises) later::run_now(1L)
+test_type("character", ip_addr())
+test_type("character", names(ip_addr()))
+test_null(write_stdout(""))
 test_true(.interrupt())
 test_true(!.interrupt(FALSE))
 test_true(!identical(get0(".Random.seed"), {.advance(); .Random.seed}))
 test_type("integer", .Call(nanonext:::rnng_traverse_precious))
+
+if (!interactive() && Sys.getenv("NOT_CRAN") == "true") {
+  test_class("conditionVariable", cv <- cv())
+  f <- file("stdin", open = "r")
+  test_true(is_nano(reader <- read_stdin()))
+  test_zero(pipe_notify(reader, cv, remove = TRUE, flag = TRUE))
+  test_true(is_aio(r <- recv_aio(reader, mode = "raw", cv = cv)))
+  close(f)
+  test_true(!wait(cv))
+  test_zero(close(reader))
+  test_equal(collect_aio(r), 7L)
+}
+
 if (Sys.info()[["sysname"]] == "Linux") {
   rm(list = ls())
-  gc()
-  .Call(nanonext:::rnng_thread_shutdown)
+  invisible(gc())
+  .Call(nanonext:::rnng_fini_priors)
   Sys.sleep(1L)
   .Call(nanonext:::rnng_fini)
   invisible()
